#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "scrollbar.h"

extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern GC large_gc;
extern XFontStruct *small_font;
extern XFontStruct *large_font;
extern GC bvl_high_gc;
extern GC bvl_low_gc;
extern GC clear_gc;
extern GC hl_gc;

struct scrollbar_list *scrollbars=NULL;

int check_scrollbars_destroyed(XEvent report)
{
   struct scrollbar_list *bdl, *pbdl=NULL;
   
   for(bdl = scrollbars; bdl; bdl = bdl->next) {
      if(bdl->data->container == report.xdestroywindow.window) {
	 if(pbdl) {
	    pbdl->next = bdl->next;
	 }
	 else {
	    scrollbars = bdl->next;
	 }
	 free(bdl->data);
	 free(bdl);
	 return 1;
      }
   }
   return 0;
}

void add_scrollbar(struct scrollbar *b)
{
   struct scrollbar_list *bdl;

   bdl = (struct scrollbar_list *)malloc(sizeof(struct scrollbar_list));
   bdl->data = b;
   
   if(scrollbars == NULL) {
      bdl->next = NULL;
      scrollbars = bdl;
   }
   else {
      bdl->next = scrollbars;
      scrollbars = bdl;
   }
}

void scrollbar_down(struct button_arg *ba) {
   ba->s->val++;

   scrollbar_bounds(ba->s);
   refresh_scrollbar(ba->s);
   ba->s->changed(ba->a, ba->s->val);
}

void scrollbar_up(struct button_arg *ba) {
   ba->s->val--;

   scrollbar_bounds(ba->s);
   refresh_scrollbar(ba->s);
   ba->s->changed(ba->a, ba->s->val);
}

void set_scrollbar(struct scrollbar *s, int val)
{
   s->val = val;
   
   scrollbar_bounds(s);
   refresh_scrollbar(s);
}

void set_scrollbar_bounds(struct scrollbar *s, int val, int min, int max, int view)
{
   s->val = val;
   s->min = min;
   s->max = max;
   s->view = view;
   
   scrollbar_bounds(s);
   refresh_scrollbar_resize(s);
}

void scrollbar_bounds(struct scrollbar *s) {
   if(s->val < s->min) {
      s->val = s->min;
   }
   else if(s->val > s->max - s->view) {
      s->val = s->max - s->view;
   }
}

void scrollbar_moved(struct button_arg *ba, int new_y)
{
   ba->s->val = ba->s->min + ((ba->s->max - ba->s->min) * new_y) / (ba->s->height-32);
   
   scrollbar_bounds(ba->s);
   refresh_scrollbar(ba->s);
   
   ba->s->changed(ba->a, ba->s->val);
}

void refresh_scrollbar(struct scrollbar *s) {
   int handle_height;
   
   if(s->max - s->min) {      
      move_scroll_handle(s->handle,
				((s->val - s->min) * (s->height-32) / (s->max - s->min)));
   }
   else {
      move_scroll_handle(s->handle, 0);
   }
   
   if(s->pixmap == 0L) {
      return;
   }

   XCopyArea(display, s->pixmap, s->container, large_gc,
	     0, 0,
	     s->width, s->height,
	     0, 0);

}

void refresh_scrollbar_resize(struct scrollbar *s) {
   int handle_height;
   
   if(s->max - s->min) {      
      handle_height = ((s->height-32) * s->view) / (s->max - s->min);
      move_resize_scroll_handle(s->handle,
				((s->val - s->min) * (s->height-32) / (s->max - s->min)),
				handle_height);
   }
   else {
      handle_height = s->height - 32;
      move_resize_scroll_handle(s->handle, 0, handle_height);
   }
   
   XCopyArea(display, s->pixmap, s->container, large_gc,
	     0, 0,
	     s->width, s->height,
	     0, 0);
}


struct scrollbar *create_scrollbar(Window parent, int x, int y,
				 int width, int height,
				 int init_val, int min, int max, int view,
				 void *changed(), void *arg)
{
   int font_height;
   struct scrollbar *s;
   struct button_arg *ba;
   int handle_height;
   
   font_height = small_font->ascent + small_font->descent;
   
   s = (struct scrollbar *)malloc(sizeof(struct scrollbar));

   s->width = width;
   s->height = height;

   if(view > (max - min)) {
      view = max - min;
      handle_height = height-32;
   }
   else {
      handle_height = ((height-32) * view) / (max-min);
   }
   
   s->val = init_val;
   s->min = min; s->max = max;
   s->view = view;
   
   s->container = create_subwindow(parent, x, y, width, height);
   
   ba = (struct button_arg *)malloc(sizeof(struct button_arg));
   ba->s = s;
   ba->a = arg;
   
   s->upbutton = create_arrow_up_button(s->container, 0, 0, width, 16,
					(void *)scrollbar_up, ba, (char *)"");

   s->downbutton = create_arrow_down_button(s->container, 0, height - 16, width, 16, (void *)scrollbar_down, ba, (char *)"");

   s->handle = create_scroll_handle(s->container, 2, 16,
				    width-4, handle_height,
				    16, height - 16, (void *)scrollbar_moved, ba);
   
   s->changed = (void *)changed;

   s->pixmap = create_scrollbar_pixmap(s->container, width, height);
   
   refresh_scrollbar(s);

   add_scrollbar(s);
   
   return s;
}

int check_scrollbars_expose(XEvent report)
{
   struct scrollbar_list *sl;
   
   for(sl = scrollbars; sl != NULL; sl = sl->next) {
      if(sl->data->container == report.xexpose.window) {
	 refresh_scrollbar(sl->data);
	 return 1;
      }
   }
   return 0;
}


Pixmap create_scrollbar_pixmap(Window window,
			       unsigned int width, unsigned int height)
{
   Pixmap pixmap;
   
   pixmap = XCreatePixmap(display, window, width, height, display_depth);
   
   XFillRectangle(display, pixmap, clear_gc, 0, 0, width, height);

   XDrawLine(display, pixmap, bvl_high_gc, 0, 0, 0, height-1);
   XDrawLine(display, pixmap, bvl_low_gc, 1, 0, 1, height-1);
   
   XDrawLine(display, pixmap, bvl_high_gc,
	     width-2, 0, width-2, height-1);
   XDrawLine(display, pixmap, bvl_low_gc,
	     width-1, 0, width-1, height-1);
   
   
   return pixmap;
}

